#!/bin/bash
# PaaS: Web-based wrapper to launch application

set -eu

SCRIPT_SELF=/opt/paas/paas.sh
SCRIPT_WRAPPER=/opt/paas/paas-runapp.sh
SCRIPT_KILL=/opt/paas/paas-killapp.sh
SCRIPT_PHPWORKER=/opt/paas/paas-writefinallogs.php
DEFAULT_BASEIMG=/opt/paas/base-current.tar.xz

fatal() {
	echo "$1" >&2
	exit 1
}

usage() {
	cat <<EOM
Usage:
  paas-runapp.sh [arguments]

Options:
  --active                       Do nothing
  --buildimage                   Build new base image file
  --builddir %dir                Build base FS in existing directory
  --execdir %dir                 Run directory (blocking)
  --exec %appzip                 Run zip (blocking) with default base image file
  --exec-with %baseimg %appzip   Run zip (blocking) with custom base image file
  --gensudo                      Display configuration for /etc/sudoers
  --help                         Display this message
  --web-inner ?? ?? ??           (internal use only)
  --web-run %appzip              (internal use only)

EOM
	exit 1
}

mark_running() {
	local workdir="$1"
	local sockfile=""

	# socktailf: first line is path to socket file

	read -r sockfile

	# if the process was very quickly aborted, sockfile may have already been destroyed at this point

	chmod 777 "$sockfile" || true
	local sockdir="$(dirname "$sockfile")"
	chmod 755 "$sockdir" || true

	echo "UPDATE running SET sockfile = '${sockfile}' WHERE localpath = '${workdir}' AND end_time = 0;" \
		| sqlite3 "$DB_LOCATION"

	# socktailf -d -x: remaining lines are the dumped buffer

	"$SCRIPT_PHPWORKER" "$DB_LOCATION" "$workdir"
}

container_setup() {
	local baseimg="$1"
	local appzip="$2"
	local workdir="$3"

	[[ -f "$baseimg" ]] || fatal "Missing base image '$baseimg'"
	[[ -f "$appzip" ]] || fatal "Missing application zip '$appzip'"
	[[ -d "$workdir" ]] || fatal "Missing working directory '$workdir'"

	#

	tar xJf "$baseimg" -C "$workdir"

	# copy in supplied app
	cp "$appzip" "${workdir}/tmp/app.zip"
	chown root:root "${workdir}/tmp/app.zip"
	chmod 666 "${workdir}/tmp/app.zip"

	# startup script handles unzip
	cat > "${workdir}/tmp/startup.sh" <<EOM
#!/bin/bash
unzip /tmp/app.zip -d /app/
cd /app/
sed -i 's/\r//' ./start
exec bash ./start
EOM

	chown root:root "${workdir}/tmp/startup.sh"
	chmod 755 "${workdir}/tmp/startup.sh"

}

container_run() {

	local workdir="$1"

	if [[ ! -d "$workdir" ]] ; then
		fatal "Expected '$workdir' to be a directory"
	fi

	# run
	# --property=MemoryLimit=128M --property=CPUQuota=10%
	# but the property argument is missing from jessie's systemd
	# simulate via systemd-run -p {prop} systemd-nspawn? seems ineffectual
	systemd-nspawn -q -D "${workdir}" -u appuser /tmp/startup.sh || true

	# we're finished with the container
}

container_teardown() {
	local workdir="$1"
	rm -r "$workdir"
}

buildbase() {

	local workdir="$1"

	if [[ ! -d "$workdir" ]] ; then
		fatal "Expected '$workdir' to be a directory"
	fi

	debootstrap --variant=minbase jessie "$workdir"

	# npm requires python2.7, php5-cli requires perl
	chroot "$workdir" bash -s <<'EOM'

apt-get -yqq install \
 nodejs nodejs-legacy npm \
 perl \
 python2.7 python-pip \
 python3 python3-pip \
 php5-cli \
 unzip \
 --no-install-recommends

useradd -k /dev/null -d /app/ -m appuser
rm /var/cache/apt/archives/*.deb

EOM
	# ok
}

require_db_location_var() {
	if [[ ! -f "$DB_LOCATION" ]] ; then
		fatal "Expected '$DB_LOCATION' file to exist (DB_LOCATION env var)"
	fi
}

main() {

	if [[ $# -lt 1 ]] ; then
		usage
	fi

	case "$1" in

		--active)
			return 0
		;;

		--buildimage)
			local workdir="$(mktemp -d)"
			buildbase "$workdir"
			tar cJf "base-$(date +%Y%m%d%H%M%S).tar.xz" -C "$workdir" .
			rm -r "$workdir"
		;;

		--builddir)
			local workdir="$2"
			buildbase "$workdir"
		;;

		--execdir)
			container_run "$2"
		;;

		--exec)
			local appzip="$2"
			local workdir="$(mktemp -d)"
			container_setup "$DEFAULT_BASEIMG" "$appzip" "$workdir"
			container_run "$workdir"
			container_teardown "$workdir"
		;;

		--exec-with)
			local baseimg="$2"
			local appzip="$3"
			local workdir="$(mktemp -d)"
			container_setup "$baseimg" "$appzip" "$workdir"
			container_run "$workdir"
			container_teardown "$workdir"
		;;

		--gensudo)
			echo "www-data ALL=NOPASSWD: ${SCRIPT_WRAPPER}"
			echo "www-data ALL=NOPASSWD: ${SCRIPT_KILL}"
		;;

		--web-run)
			require_db_location_var
			local appzip="$2"
			local workdir="$(mktemp -d)"
			echo "$workdir"
			nohup \
				"$SCRIPT_SELF" --web-inner "$DEFAULT_BASEIMG" "$appzip" "$workdir" \
				>"${workdir}/stdout.log" 2>"${workdir}/stderr.log" &
		;;

		--web-inner)
			require_db_location_var
			local baseimg="$2"
			local appzip="$3"
			local workdir="$4"

			container_setup "$baseimg" "$appzip" "$workdir"
			rm "$appzip" # it was copied for us by PHP
			( container_run "$workdir" 2>&1 ) | socktailf -d -x | mark_running "$workdir"

			container_teardown "$workdir"

		;;

		--help)
			usage
		;;

		*)
			usage
		;;

	esac

	return 0
}

main "$@"
