<?php

function api__public_projectlist($token) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	
	return [
		'projects' => DB::project_list($user_id),
		'invite_key' => DB::user_getkey($user_id),
	];
}

function api__public_projectnew($token, $title) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	return DB::project_init($user_id, $title);
}

function api__public_projectdelete($token, $project_id) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	api__requireowns($user_id, $project_id);
	
	// Check if any instances are still running - they should be stopped first
	if (DB::running_countalivefor($project_id) === 0) {	
		DB::project_unregister($project_id);
	} else {
		return [ 'error' => 'There are still running instances. Please stop them before deregistering the project.' ];
	}
		
	return 'ok';
}

function api__public_projectinfo($token, $project_id) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	api__requireowns($user_id, $project_id);
	
	return [
		'title'     => DB::project_getname($project_id),
		'api_key'   => DB::project_getapikey($project_id),
		'versions'  => DB::appdata_findallfor($project_id),
		'instances' => DB::running_overview($project_id),
	];
}

function api__public_register($invite_key, $username, $password) {
	$invitee = DB::user_lookup_invitekey($invite_key);
	if (! $invitee) {
		return [ 'error' => 'Unknown invitation code.' ];
	}
	
	DB::user_regenkey($invitee);
	
	if (! DB::user_register($username, $password, $invitee)) {
		return [ 'error' => 'Couldn\'t create user. Try another username?' ];
	}
	
	return api__public_login($username, $password);
}

function api__public_login($username, $password) {
	$attempt = DB::user_attemptlogin($username, $password);
	
	if ($attempt === false) {
		return [ 'error' => 'Invalid credentials, please retry' ];
	} else {
		return [ 'authtoken' => $attempt ];
	}
}

function api__public_logout($token) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	DB::user_settoken('', $user_id);
	return 'ok';
}

function api__public_regenerate($token, $project_id) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	api__requireowns($user_id, $project_id);
	DB::project_apikey_regenerate($project_id);
	return 'ok';
}

function api__public_upload($token, $project_id) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	api__requireowns($user_id, $project_id);
	
	if (
		isset($_FILES['appdata']) &&
		$_FILES['appdata']['error'] === 0 &&
		$_FILES['appdata']['size'] > 0
	) {
		
		$validate = paas__validate($_FILES['appdata']['tmp_name']);
		if ($validate === true) {
			
			$data = file_get_contents($_FILES['appdata']['tmp_name']);
			$appdata_id = DB::appdata_add($project_id, $data);
			return $appdata_id;
		
		} else {
			return ['error' => 'Bad upload: '.$validate];
		}
		
	} else {
		return ['error' => 'Failed to upload file.'];
	}
}

function api__public_download($token, $version_id) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	api__requireownsappdata($user_id, $version_id);
	
	header('Content-Type: application/zip');
	header('Content-Disposition: attachment ; filename="paas-application-'.sha1($version_id).'.zip"');
	
	die(DB::appdata_getdata($version_id));
}

function api__public_exec($token, $appdata_id) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	api__requireownsappdata($user_id, $appdata_id);
	
	paas__start($appdata_id);
	return 'ok';
}

function api__public_kill($token, $running_id) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	api__requireownsrunning($user_id, $running_id);
	
	$row = DB::running_getlogdata($running_id);

	if ($row['end_time'] != '0') {
		return [ 'error' => 'Instance already stopped.' ];
	}
	
	if ( paas__kill($row['localpath']) ) {
		return 'ok';
	} else {
		return [ 'error' => 'Invalid request.' ];
	}
}

function api__public_logs($token, $running_id) {
	api__requirevalid($user_id = DB::user_findbytoken($token));
	api__requireownsrunning($user_id, $running_id);
	
	$row = DB::running_getlogdata($running_id);
	
	if ($row['end_time'] != '0') {
		return ($row['finaloutput'] ?: '');
		
	} elseif (strlen($row['sockfile'])) {
		$ret = shell_exec('nc -U '.escapeshellarg($row['sockfile']));
		if (! is_string($ret)) {
			return '(Internal Error)';
		} else {
			return $ret;
		}
		
	} else {
		return '(Still initialising, please wait)';
	}
}

// ##

function api__requirevalid($token) {
	if ($token === false) {
		die(json_encode(['error' => 'Session expired']));
	}
}

function api__requireowns($user_id, $project_id) {
	if (! DB::project_isownedby($user_id, $project_id)) {
		die(json_encode(['error' => 'Invalid project selection']));
	}
}

function api__requireownsappdata($user_id, $appdata_id) {
	if (! DB::appdata_isownedby($user_id, $appdata_id)) {
		die(json_encode(['error' => 'Invalid version selection']));
	}
}

function api__requireownsrunning($user_id, $running_id) {
	if (! DB::running_isownedby($user_id, $running_id)) {
		die(json_encode(['error' => 'Invalid instance selection']));
	}
}

/**
 * Controller
 */
function api__processRequest() {
	
	header('Content-Type: application/json');

	if (array_key_exists('action', $_POST)) {
		$fn = 'api__public_'.$_POST['action'];
		
		if (function_exists($fn)) {
			
			$fn_ref = new ReflectionFunction($fn);
			$supply = [];
			
			foreach($fn_ref->getParameters() as $param) {
				$param_name = $param->getName();
				if (! isset( $_POST[$param_name])) {
					die(json_encode(['error' => 'Invalid request']));
				}
				$supply[] = $_POST[$param_name];
			}
			$ret = call_user_func_array($fn, $supply);
			die(json_encode($ret));
		} else {
			die(json_encode(['error' => 'Invalid request']));
		}
	} else {
		die(json_encode(['error' => 'Invalid request']));
	}

}
