<?php

class DB {
	
	protected function __construct() { } 
	
	protected static function get() {
		static $db = null;
		if (is_null($db)) {
			$db = new \PDO('sqlite:../paas.db');
			
			$db->query('
				CREATE TABLE IF NOT EXISTS users (
					id INTEGER PRIMARY KEY AUTOINCREMENT,
					username VARCHAR,
					pass_hash VARCHAR,
					session VARCHAR,
					invite_key VARCHAR,
					invited_by INTEGER
				);
			');
			$db->query('
				CREATE TABLE IF NOT EXISTS projects (
					id INTEGER PRIMARY KEY AUTOINCREMENT,
					user_id INTEGER,
					visible INTEGER,
					title VARCHAR,
					api_key VARCHAR
				);
			');
			$db->query('
				CREATE TABLE IF NOT EXISTS appdata (
					id INTEGER PRIMARY KEY AUTOINCREMENT,
					project_id INTEGER,
					upload_time INTEGER,
					data BLOB
				);
			');
			$db->query('
				CREATE TABLE IF NOT EXISTS running (
					id INTEGER PRIMARY KEY AUTOINCREMENT,
					appdata_id INTEGER,
					localpath VARCHAR,
					sockfile VARCHAR,
					finaloutput VARCHAR,
					start_time INTEGER,
					end_time INTEGER
				);
			');
		}
		return $db;
	}
	
	public static function query($sql, $params=[]) {
		$stmt = self::get()->prepare($sql);
		if ($stmt === false) {
			var_export( self::get()->errorInfo() );
			// invalid sql
		}
		$stmt->execute($params);
		return $stmt->fetchAll(\PDO::FETCH_ASSOC);
	}
	
	/**
	 * user_*
	 */
	
	public static function user_exists($username) {
		$info = self::query(
			'SELECT id FROM users WHERE username = ?',
			[ $username ]
		);
		
		return ( count($info) === 1 ) ? intval($info[0]['id']) : false;
	}
	
	public static function user_register($username, $password, $invited_by=0) {
		if (self::user_exists($username)) {
			return false;
		}
		self::query(
			'INSERT INTO users (username, pass_hash, session, invite_key, invited_by) VALUES (?, ?, ?, ?, ?)',
			[$username, sha1($password), '', sha1(uniqid()), intval($invited_by) ]
		);
		return true;
	}
	
	public static function user_getkey($user_id) {
		$info = self::query(
			'SELECT invite_key FROM users WHERE id = ?', 
			[ $user_id ]
		);
		
		return ( count($info) === 1 ) ? $info[0]['invite_key'] : false;
	}
	
	public static function user_regenkey($user_id) {
		self::query(
			'UPDATE users SET invite_key = ? WHERE id = ?', 
			[ sha1(uniqid()), $user_id ]
		);
	}
	
	public static function user_lookup_invitekey($invitekey) {
		$info = self::query(
			'SELECT id FROM users WHERE invite_key = ?', 
			[ $invitekey ]
		);
		
		return ( count($info) === 1 ) ? intval($info[0]['id']) : false;
	}
	
	public static function user_attemptlogin($username, $password) {
		$pass_hash = sha1($password);
		$info = self::query(
			'SELECT * FROM users WHERE username = ? AND pass_hash = ?',
			[$username, $pass_hash]
		);
		
		if (count($info) === 1) {
			$token = sha1(uniqid());
			self::user_settoken( $info[0]['id'], $token );
			return $token;
			
		} else {
			return false; // not existent
			
		}
	}
	
	public static function user_setpass($user_id, $password) {
		self::query(
			'UPDATE users SET pass_hash = ? , token = ? WHERE id = ?',
			[ sha1($password), '', $user_id ]
		);
	}
	
	public static function user_settoken($user_id, $token) {
		self::query(
			'UPDATE users SET session = ? WHERE id = ?',
			[ $token, $user_id ]
		);
	}
	
	public static function user_findbytoken($token) {
		// the default token and the logged-out token are the empty
		//   string - don't allow them to be valid responses!
		
		if (! strlen(trim($token))) {
			return false;
		}
		
		$info = self::query(
			'SELECT id FROM users WHERE session = ?',
			[$token]
		);
		if (count($info) === 1) {
			return intval($info[0]['id']);
		} else {
			return false;
		}
	}
	
	/**
	 * project_*
	 */
	
	public static function project_init($user_id, $title) {
		$info = self::query(
			'INSERT INTO projects (user_id, visible, title, api_key) VALUES (?, ?, ?, ?)',
			[ $user_id, 1, $title, sha1(uniqid()) ]
		);
		return self::get()->lastInsertId();
	}
	
	public static function project_list($user_id) {
		$results = self::query(
			'SELECT id, title FROM projects WHERE user_id = ? AND visible = ?',
			[ $user_id, 1 ]
		);
		$ret = [];
		foreach($results as $row) {
			$ret[$row['id']] = $row['title'];
		}
		return $ret;
	}
	
	public static function project_unregister($project_id) {
		self::query(
			'UPDATE projects SET visible = ? WHERE id = ?',
			[ 0, $project_id ]
		);
	}
	
	public static function project_isownedby($user_id, $project_id) {
		$ret = self::query(
			'SELECT id FROM projects WHERE id = ? AND user_id = ? AND visible = ?',
			[ $project_id, $user_id, 1 ]
		);
		return (count($ret) === 1);
	}
	
	public static function project_apikey_regenerate($project_id) {
		self::query(
			'UPDATE projects SET api_key = ? WHERE id = ?',
			[ sha1(uniqid()), $project_id ]
		);
	}
	
	public static function project_apikey_lookup($apikey) {
		$info = self::query(
			'SELECT id FROM projects WHERE api_key = ?',
			[ $apikey ]
		);
		return ( (count($info) === 1) ? intval($info[0]['id']) : false );
	}
	
	public static function project_getapikey($project_id) {
		$ret = self::query(
			'SELECT api_key FROM projects WHERE id = ?',
			[ $project_id ]
		);
		if (count($ret) === 1) {
			return $ret[0]['api_key'];
		} else {
			return false;
		}
	}
	
	public static function project_getname($project_id) {
		$ret = self::query(
			'SELECT title FROM projects WHERE id = ?',
			[ $project_id ]
		);
		if (count($ret) === 1) {
			return $ret[0]['title'];
		} else {
			return false;
		}
	}
	
	public static function project_setname($project_id, $title) {
		self::query(
			'UPDATE projects SET title = ? WHERE id = ?',
			[ $title, $project_id ]
		);
	}
	
	
	/**
	 * appdata_*
	 */
	
	public static function appdata_isownedby($user_id, $appdata_id) {
		$search = self::query('
			SELECT
				users.id
			FROM
				appdata
				JOIN projects ON appdata.project_id = projects.id
				JOIN users ON projects.user_id = users.id
			WHERE
				appdata.id = ? AND
				users.id = ?
			',
			[ $appdata_id, $user_id ]
		);
		return ( count($search) === 1 );
	}
	
	public static function appdata_findallfor($project_id) {
		return self::query('
			SELECT
				appdata.id,
				appdata.upload_time,
				((LENGTH(appdata.data) * 3) / 4) filesize,
				CASE 
					WHEN runct.ct IS NULL THEN 0
					ELSE runct.ct
				END AS running_count
			FROM
				appdata
				LEFT OUTER JOIN (
					SELECT
						appdata_id,
						COUNT(*) ct
					FROM
						running
					WHERE
						end_time = 0
					GROUP BY
						appdata_id
				) runct ON runct.appdata_id = appdata.id
			WHERE
				appdata.project_id = ?
			ORDER BY
				appdata.id DESC
			',
			[ $project_id ]
		);
	}

	public static function appdata_add($project_id, $data) {
		self::query(
			'INSERT INTO appdata ( project_id, upload_time, data ) VALUES ( ?, ?, ? )',
			[ $project_id, time(), base64_encode($data) ]
		);
		return self::get()->lastInsertId();
	}
	
	public static function appdata_getdata($appdata_id) {
		$ret = self::query(
			'SELECT data FROM appdata WHERE id = ?',
			[ $appdata_id ]
		);
		if (count($ret) === 1) {
			return base64_decode($ret[0]['data']);
		} else {
			return false;
		}
	}
	
	/**
	 * running_*
	 */
	
	public static function running_start($appdata_id, $localpath) {
		self::query(
			'INSERT INTO running ( appdata_id, localpath, start_time, end_time ) VALUES ( ?, ?, ?, ? )',
			[ $appdata_id, $localpath, time(), 0 ]
		);
	}
	
	public static function running_stop($localpath) {
		self::query(
			'UPDATE running SET end_time = ? WHERE localpath = ? AND end_time = ?',
			[ time(), $localpath, 0 ]
		);
	}
	
	public static function running_isownedby($user_id, $running_id) {
		$check = self::query(
			'SELECT
				1
			FROM
				running
				JOIN appdata ON running.appdata_id = appdata.id
				JOIN projects ON appdata.project_id = projects.id
			WHERE
				running.id = ? AND
				projects.user_id = ?
			',
			[ $running_id, $user_id ]
		);
		
		return ( count($check) === 1 );
	}
	
	public static function running_countalivefor($project_id) {
		$info = self::query(
			'SELECT
				COUNT(*) ct
			FROM
				running
				JOIN appdata ON running.appdata_id = appdata.id
			WHERE
				appdata.project_id = ? AND
				running.end_time = ?
			',
			[ $project_id, 0 ]
		);
		return ( count($info) === 1 ? intval($info[0]['ct']) : 0 );
	}
	
	public static function running_getlogdata($running_id) {
		$check = self::query('SELECT * FROM running WHERE id = ?', [$running_id]);
		if (count($check) === 1) {
			return $check[0];
		} else {
			return false;
		}
	}
	
	public static function running_overview($project_id) {
		return self::query(
			'SELECT
				running.id,
				running.appdata_id,
				running.start_time,
				running.end_time,
				CASE WHEN LENGTH(running.sockfile) > 0 THEN 1 ELSE 0 END has_started
			FROM
				running
				JOIN appdata ON running.appdata_id = appdata.id
			WHERE
				appdata.project_id = ?
			ORDER BY
				CASE WHEN running.end_time == 0 THEN 0 ELSE 1 END ASC,
				running.start_time DESC
			',
			[ $project_id ]
		);
	}
}
